import React, { useEffect, useState } from 'react';
import FullCalendar, { EventInput } from '@fullcalendar/react';
import dayGridPlugin from '@fullcalendar/daygrid';
import timeGridPlugin from '@fullcalendar/timegrid';
import interactionPlugin from '@fullcalendar/interaction';
import '../styles/Calendar.css';
import { BeamtimesService, DewarsService, PucksService } from '../../openapi';

// Define colors for each beamline
const beamlineColors: { [key: string]: string } = {
    X06SA: '#FF5733',
    X10SA: '#33FF57',
    X06DA: '#3357FF',
    Unknown: '#CCCCCC', // Gray color for unknown beamlines
};

// Custom event interface
interface CustomEvent extends EventInput {
    beamline: string;
    beamtime_shift: string;
    isSubmitted?: boolean; // Track if information is submitted
}

// Define experiment modes
const experimentModes = ['SDU-Scheduled', 'SDU-queued', 'Remote', 'In-person'];

// Utility function to darken a hex color
const darkenColor = (color: string, percent: number): string => {
    const num = parseInt(color.slice(1), 16); // Convert hex to number
    const amt = Math.round(2.55 * percent); // Calculate amount to darken
    const r = (num >> 16) + amt; // Red
    const g = (num >> 8 & 0x00FF) + amt; // Green
    const b = (num & 0x0000FF) + amt; // Blue

    // Ensure values stay within 0-255 range
    const newColor = (0x1000000 + (r < 255 ? (r < 0 ? 0 : r) : 255) * 0x10000 + (g < 255 ? (g < 0 ? 0 : g) : 255) * 0x100 + (b < 255 ? (b < 0 ? 0 : b) : 255)).toString(16).slice(1);
    return `#${newColor}`;
};

const Calendar: React.FC = () => {
    const [events, setEvents] = useState<CustomEvent[]>([]);
    const [isLoading, setIsLoading] = useState(false);
    const [fetchError, setFetchError] = useState<string | null>(null);
    const [selectedEventId, setSelectedEventId] = useState<string | null>(null);
    const [eventDetails, setEventDetails] = useState<CustomEvent | null>(null);
    const [eventAssociations, setEventAssociations] = useState<{ [eventId: string]: {dewars: string[], pucks: string[]} }>({});
    const [userDetails, setUserDetails] = useState({
        name: '',
        firstName: '',
        phone: '',
        email: '',
        extAccount: '',
        experimentMode: experimentModes[0],
    });
    const [shipments, setShipments] = useState<any[]>([]); // State for shipments
    const [selectedDewars, setSelectedDewars] = useState<string[]>([]); // Track selected dewars for the experiment

    const fetchBeamtimes = async () => {
        setIsLoading(true);
        setFetchError(null);

        try {
            const beamtimeData = await BeamtimesService.getMyBeamtimesProtectedBeamtimesMyBeamtimesGet(); // Replace with actual API function
            const formattedEvents: CustomEvent[] = beamtimeData.map((beamtime: any) => ({
                id: `${beamtime.beamline}-${beamtime.start_date}`,
                title: `${beamtime.beamline}: ${beamtime.shift}`,
                start: beamtime.start_date,
                end: beamtime.end_date,
                beamtime_id: beamtime.id,
                beamline: beamtime.beamline || 'Unknown',
                beamtime_shift: beamtime.shift || 'Unknown',
                backgroundColor: beamlineColors[beamtime.beamline] || beamlineColors.Unknown,
                borderColor: '#000',
                textColor: '#fff',
            }));

            setEvents(formattedEvents);
        } catch (error) {
            console.error('Failed to fetch beamtime data:', error);
            setFetchError('Failed to load beamtime data. Please try again later.');
        } finally {
            setIsLoading(false);
        }
    };

    // Fetch beamtimes on component mount
    useEffect(() => {
        fetchBeamtimes();
    }, []);

    useEffect(() => {
        // Only fetch if you want to update dewars when eventDetails change,
        // or run only on initial mount by passing [] as deps
        if (eventDetails) {
            const fetchDewars = async () => {
                try {
                    // This name depends on your codegen's operationId / function name!
                    const dewars = await DewarsService.getRecentDewarsWithPucks();
                    setShipments(dewars);
                } catch (err) {
                    // Optionally handle error
                    console.error("Failed to fetch dewars with pucks", err);
                }
            };
            fetchDewars();
        }
    }, [eventDetails]); // Fetch dewars when an event is selected


    //const fetchShipments = async () => {
    //        try {
    //            const response = await fetch('/shipmentdb.json');
    //
    //            // Check for HTTP errors
    //            if (!response.ok) {
    //                throw new Error(`HTTP error! status: ${response.status}`);
    //            }
    //
    //            // Parse the JSON response
    //            const data = await response.json();
    //
    //            const availableDewars: any[] = [];
    //
    //            data.shipments.forEach(shipment => {
    //                if (shipment.shipment_status === "In Transit") {
    //                    shipment.dewars.forEach(dewar => {
    //                        if (dewar.shippingStatus === "shipped" && dewar.returned === "") {
    //                            availableDewars.push(dewar);
    //                        }
    //                    });
    //                }
    //            });
    //
    //            console.log('Available Dewars:', availableDewars);
    //            setShipments(availableDewars);
    //        } catch (error) {
    //            console.error('Error fetching shipments:', error);
    //            // Optionally display the error to the user in the UI
    //        }
    //    };
    //
    //    fetchEvents();
    //    fetchShipments();
    //}, []);

    const handleEventClick = (eventInfo: any) => {
        const clickedEventId = eventInfo.event.id;
        setSelectedEventId(clickedEventId);

        const selectedEvent = events.find(event => event.id === clickedEventId) || null;
        setEventDetails(selectedEvent);
    };

    const handleInputChange = (e: React.ChangeEvent<HTMLInputElement | HTMLSelectElement>) => {
        const { name, value } = e.target;
        setUserDetails(prevDetails => ({
            ...prevDetails,
            [name]: value,
        }));
    };


    const handleSubmit = (e: React.FormEvent) => {
        e.preventDefault();

        if (eventDetails) {
            const updatedEvents = events.map(event =>
                event.id === eventDetails.id
                    ? { ...event, isSubmitted: true, selectedDewars } // Associate selected dewars
                    : event
            );
            setEvents(updatedEvents);
        }

        console.log('User Details:', userDetails);
        console.log('Selected Dewars:', selectedDewars);

        // Reset user details and selected dewars after submission
        setUserDetails({
            name: '',
            firstName: '',
            phone: '',
            email: '',
            extAccount: '',
            experimentMode: experimentModes[0],
        });
        setSelectedDewars([]); // Reset selected dewars
    };
    const assignDewarToBeamtime = async (dewarId: number, beamtimeId: number) => {
        return DewarsService.assignDewarToBeamtime(dewarId, beamtimeId);
    };

    const unassignDewarFromBeamtime = async (dewarId: number) => {
        // Pass "0" as the special value (if that's how you unassign)
        return DewarsService.assignDewarToBeamtime(dewarId, 0); // or whatever your backend supports
    };

    const assignPuckToBeamtime = async (puckId: number, beamtimeId: number) => {
        return PucksService.assignPuckToBeamtime(puckId, beamtimeId);
    };

// For unassignment (beamtime_id = 0)
    const unassignPuckFromBeamtime = async (puckId: number) => {
        return PucksService.assignPuckToBeamtime(puckId, 0);
    };



    function handleDewarAssignment(dewarId: string) {
        const event = events.find(e => e.id === selectedEventId);
        if (!event) return;
        const beamtimeId = event.beamtime_id;
        if (beamtimeId == null) {
            console.error("No numeric beamtime_id found for selected event:", event);
            return;
        }

        // Find out if it's already assigned to this shift!
        const prev = eventAssociations[selectedEventId] || { dewars: [], pucks: [] };
        const isAssigned = prev.dewars.includes(dewarId);

        if (!isAssigned) {
            // Assign it immediately
            assignDewarToBeamtime(Number(dewarId), Number(beamtimeId))
                .then(() => {
                    setEventAssociations(prevAssoc => {
                        const updated = prevAssoc[selectedEventId] || { dewars: [], pucks: [] };
                        return {
                            ...prevAssoc,
                            [selectedEventId]: {
                                ...updated,
                                dewars: [...updated.dewars, dewarId]
                            }
                        };
                    });
                })
                .catch(e => {
                    console.error("Failed to assign dewar to beamtime", e);
                });
        } else {
            // Unassign (patch to None) immediately
            unassignDewarFromBeamtime(Number(dewarId))
                .then(() => {
                    setEventAssociations(prevAssoc => {
                        const updated = prevAssoc[selectedEventId] || { dewars: [], pucks: [] };
                        return {
                            ...prevAssoc,
                            [selectedEventId]: {
                                ...updated,
                                dewars: updated.dewars.filter(id => id !== dewarId)
                            }
                        };
                    });
                })
                .catch(e => {
                    console.error("Failed to unassign dewar from beamtime", e);
                });
        }
    }


    function handlePuckAssignment(puckId: string) {
        const event = events.find(e => e.id === selectedEventId);
        if (!event) return;
        const beamtimeId = event.beamtime_id;
        if (beamtimeId == null) {
            console.error("No numeric beamtime_id found for selected event:", event);
            return;
        }

        // Find out if it's already assigned to this shift!
        const prev = eventAssociations[selectedEventId] || { dewars: [], pucks: [] };
        const isAssigned = prev.pucks.includes(puckId);

        if (!isAssigned) {
            // Assign it immediately
            assignPuckToBeamtime(Number(puckId), Number(beamtimeId))
                .then(() => {
                    setEventAssociations(prevAssoc => {
                        const updated = prevAssoc[selectedEventId] || { dewars: [], pucks: [] };
                        return {
                            ...prevAssoc,
                            [selectedEventId]: {
                                ...updated,
                                pucks: [...updated.pucks, puckId]
                            }
                        };
                    });
                })
                .catch(e => {
                    console.error("Failed to assign puck to beamtime", e);
                });
        } else {
            // Unassign (patch to None) immediately
            unassignPuckFromBeamtime(Number(puckId))
                .then(() => {
                    setEventAssociations(prevAssoc => {
                        const updated = prevAssoc[selectedEventId] || { dewars: [], pucks: [] };
                        return {
                            ...prevAssoc,
                            [selectedEventId]: {
                                ...updated,
                                pucks: updated.pucks.filter(id => id !== puckId)
                            }
                        };
                    });
                })
                .catch(e => {
                    console.error("Failed to unassign puck from beamtime", e);
                });
        }
    }

    function findAssociatedEventForDewar(dewarId: string): string | undefined {
        for (const [eventId, assoc] of Object.entries(eventAssociations)) {
            if (assoc.dewars.includes(dewarId)) return eventId;
        }
    }
    function findAssociatedEventForPuck(puckId: string): string | undefined {
        for (const [eventId, assoc] of Object.entries(eventAssociations)) {
            if (assoc.pucks.includes(puckId)) return eventId;
        }
    }


    const eventContent = (eventInfo: any) => {
        const beamline = eventInfo.event.extendedProps.beamline || 'Unknown';
        const isSelected = selectedEventId === eventInfo.event.id;
        const isSubmitted = eventInfo.event.extendedProps.isSubmitted;
        const hasAssociations =
            eventAssociations[eventInfo.event.id]?.dewars.length > 0 ||
            eventAssociations[eventInfo.event.id]?.pucks.length > 0;


        const backgroundColor = isSubmitted
            ? darkenColor(beamlineColors[beamline] || beamlineColors.Unknown, -20)
            : isSelected
                ? '#FFD700'
                : (beamlineColors[beamline] || beamlineColors.Unknown);

        return (
            <div
                style={{
                    backgroundColor: backgroundColor,
                    color: 'white',
                    border: isSelected ? '2px solid black' : 'none',
                    borderRadius: '3px',
                    display: 'flex',
                    justifyContent: 'center',
                    alignItems: 'center',
                    height: '100%',
                    width: '100%',
                    cursor: 'pointer',
                    overflow: 'hidden',
                    boxSizing: 'border-box',
                }}
            >
                {eventInfo.event.title}
                {hasAssociations && <span style={{marginLeft: 8}}>🧊</span>}
            </div>
        );
    };

    return (
        <div className="calendar-container">
            <h2>Beamline Calendar</h2>
            <FullCalendar
                plugins={[dayGridPlugin, timeGridPlugin, interactionPlugin]}
                initialView="dayGridMonth"
                events={events}
                eventContent={eventContent}
                height={700}
                headerToolbar={{
                    left: 'prev,next',
                    center: 'title',
                    right: 'dayGridMonth',
                }}
                eventClick={handleEventClick}
            />

            {eventDetails && (
                <div className="event-details">
                    <h3>Event Details</h3>
                    <p><strong>Beamline:</strong> {eventDetails.beamline}</p>
                    <p><strong>Shift:</strong> {eventDetails.beamtime_shift}</p>

                    <h4>Select Dewars</h4>
                    <ul>
                        {shipments.map(dewar => {
                            // Are *all* pucks assigned to this event? (Assigned at Dewar level)
                            const thisEvent = eventAssociations[selectedEventId] || {dewars: [], pucks: []};
                            const dewarAssigned = thisEvent.dewars.includes(dewar.id);
                            const puckAssignments = thisEvent.pucks;

                            return (
                                <li key={dewar.id}>
                                    <label>
                                        <input
                                            type="checkbox"
                                            checked={dewarAssigned}
                                            onChange={() => handleDewarAssignment(dewar.id)}
                                        />
                                        <b>{dewar.dewar_name}</b>
                                    </label>
                                    {!dewarAssigned && dewar.pucks && dewar.pucks.length > 0 && (
                                        <ul>
                                            {(dewar.pucks || []).map(puck => {
                                                const isAssigned =
                                                    !!eventAssociations[selectedEventId]?.pucks.includes(puck.id);

                                                return (
                                                    <div key={puck.id} style={{ display: 'flex', alignItems: 'center' }}>
                                                        <span>{puck.name}</span>
                                                        <button
                                                            type="button"
                                                            style={{
                                                                marginLeft: 8,
                                                                background: isAssigned ? '#4CAF50' : '#e0e0e0',
                                                                color: isAssigned ? 'white' : 'black',
                                                                border: isAssigned ? '1px solid #388e3c' : '1px solid #bdbdbd',
                                                                borderRadius: 4,
                                                                padding: '4px 10px',
                                                                cursor: 'pointer',
                                                                transition: 'background 0.2s',
                                                            }}
                                                            onClick={() => handlePuckAssignment(dewar.id, puck.id)}
                                                        >
                                                            {isAssigned ? puck.puck_name : puck.puck_name}
                                                        </button>
                                                        {/* Optionally show assignment info, as before */}
                                                        {findAssociatedEventForPuck(puck.id) && (
                                                            <span style={{marginLeft: 8, color: 'green'}}>← Assigned to shift: {findAssociatedEventForPuck(puck.id)}</span>
                                                        )}
                                                    </div>
                                                );
                                            })}

                                        </ul>
                                    )}
                                    {/* Show associated shift, if any */}
                                    {findAssociatedEventForDewar(dewar.id) && (
                                        <span style={{marginLeft: 8, color: 'green'}}>← Assigned to shift: {findAssociatedEventForDewar(dewar.id)}</span>
                                    )}
                                </li>
                            );
                        })}
                    </ul>

                    <h4>User Information</h4>
                    <form onSubmit={handleSubmit}>
                        <label>
                            Name:
                            <input
                                type="text"
                                name="name"
                                value={userDetails.name}
                                onChange={handleInputChange}
                            />
                        </label>
                        <br />
                        <label>
                            First Name:
                            <input
                                type="text"
                                name="firstName"
                                value={userDetails.firstName}
                                onChange={handleInputChange}
                            />
                        </label>
                        <br />
                        <label>
                            Phone:
                            <input
                                type="text"
                                name="phone"
                                value={userDetails.phone}
                                onChange={handleInputChange}
                            />
                        </label>
                        <br />
                        <label>
                            Email:
                            <input
                                type="email"
                                name="email"
                                value={userDetails.email}
                                onChange={handleInputChange}
                            />
                        </label>
                        <br />
                        <label>
                            External Account:
                            <input
                                type="text"
                                name="extAccount"
                                value={userDetails.extAccount}
                                onChange={handleInputChange}
                            />
                        </label>
                        <br />
                        <label>
                            Experiment Mode:
                            <select
                                name="experimentMode"
                                value={userDetails.experimentMode}
                                onChange={handleInputChange}
                            >
                                {experimentModes.map(mode => (
                                    <option key={mode} value={mode}>{mode}</option>
                                ))}
                            </select>
                        </label>
                        <br />
                        <button type="submit">Submit</button>
                    </form>
                </div>
            )}
        </div>
    );
};

export default Calendar;
